package risc;

public class CodeWriter {

    public CodeWriter() {
        this(RISC.opcode_mnemonic, RISC.register_name);
    }

    public CodeWriter(String[] mnemonics, String[] registers) {
        this.mnemonics = mnemonics;
        this.mnemonics_width = width(mnemonics);
        this.registers = registers;
        this.registers_width = width(registers);
    }

    public String getInstructionString(int instruction) {
        StringBuffer buffer = new StringBuffer();
        appendInstruction(buffer, instruction);
        return buffer.toString();
    }

    public String getOpcodeMnemonic(int opcode) {
        return mnemonics[opcode];
    }

    public String getRegisterName(int register) {
        return registers[register];
    }

    public void appendInstruction(StringBuffer buffer, int instruction) {
        int opcode = RISC.Helper.op(instruction);
        appendOpcode(buffer, opcode, true);

        int format = RISC.opcode_format[opcode];
        switch (format) {

        case RISC.OpRaRbRc: case RISC.OpRaRbIc: case RISC.OpRaRbUc:
        case RISC.OpRa__Rc: case RISC.OpRa__Ic: case RISC.OpRa__Uc:
        case RISC.OpRa__Oc: case RISC.OpRa____:
            buffer.append(' ');
            appendRegister(buffer, RISC.Helper.ra(instruction), true);
            break;
        }

        switch (format) {

        case RISC.OpRaRbRc: case RISC.OpRaRbIc: case RISC.OpRaRbUc:
            buffer.append(' ');
            appendRegister(buffer, RISC.Helper.rb(instruction), true);
            break;
        }

        switch (format) {

        case RISC.OpRaRbRc: case RISC.OpRa__Rc: case RISC.Op____Rc:
            buffer.append(' ');
            appendRegister(buffer, RISC.Helper.rc(instruction), true);
            break;

        case RISC.OpRaRbIc: case RISC.OpRa__Ic:
            buffer.append(' ');
            appendSigned(buffer, RISC.Helper.ic(instruction));
            break;

        case RISC.OpRaRbUc: case RISC.OpRa__Uc:
            buffer.append(' ');
            appendUnsigned(buffer, RISC.Helper.uc(instruction));
            break;

        case RISC.OpRa__Oc: case RISC.Op____Oc:
            buffer.append(' ');
            appendOffset(buffer, RISC.Helper.oc(instruction));
            break;

        case RISC.Op____Lc:
            buffer.append(' ');
            appendLabel(buffer, RISC.Helper.lc(instruction));
            break;
        }
    }

    public void appendOpcode(StringBuffer buffer, int opcode, boolean pad){
        String string = mnemonics[opcode];
        buffer.append(string);
        if (pad) pad(buffer, mnemonics_width - string.length());
    }

    public void appendRegister(StringBuffer buffer, int register, boolean pad){
        String string = registers[register];
        buffer.append(string);
        if (pad) pad(buffer, registers_width - string.length());
    }

    public void appendSigned(StringBuffer buffer, int value) {
        buffer.append(value);
    }

    public void appendUnsigned(StringBuffer buffer, int value) {
        buffer.append(value);
    }

    public void appendOffset(StringBuffer buffer, int value) {
        buffer.append(value);
    }

    public void appendLabel(StringBuffer buffer, int value) {
        buffer.append(value);
    }

    public void pad(StringBuffer buffer, int width) {
        while (--width >= 0) buffer.append(' ');
    }

    private static int width(String[] strings) {
        int width = 0;
        for (int i = 0; i < strings.length; i++) {
            String string = strings[i];
            if (string != null) width = Math.max(width, string.length());
        }
        return width;
    }

    private final String[] mnemonics;
    private final int mnemonics_width;
    private final String[] registers;
    private final int registers_width;
}
