package risc.gui;

/**
 * Display of the assembly code
 *
 * 01.05.01 by Benjamin Leroy-Beaulieu
 *
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.*;
import javax.swing.text.BadLocationException;

import javax.swing.table.AbstractTableModel;
import risc.CodeWriter;
import risc.debugger.Debugger;

public class AsmCode {

    public JFrame         frame;

    private JScrollPane    codeScrollPane;
    private String[]       columnNames;
    private JTable         codeJTable;

    /** Pointer to the main gui */
    private StartGUI       startGUI;
   
    public AsmCode(StartGUI thisStartGUI) {

	startGUI = thisStartGUI;

	/** Creating the frame */
	Toolkit tk  = Toolkit.getDefaultToolkit();
	Dimension d = tk.getScreenSize();
	int screenw = d.width;
	int screenh = d.height;
	frame = new JFrame("ASM Code");
	frame.setSize(3*screenw/10, screenh);
	frame.setLocation(7*screenw/10, 0);

	/** Allowing window closing */
        frame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
	
	/** frame's layout Manager */
	frame.getContentPane().setLayout(new GridBagLayout());

	/** frame's constraint object */ 
        GridBagConstraints constraints = new GridBagConstraints();
	// components grow in both directions
        constraints.fill    = GridBagConstraints.BOTH; 
	// 5-pixel margin on all sides
        constraints.insets  = new Insets(5,5,5,5); 
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;

	/** Now, add the buttonPane to the main frame */
        constraints.fill    = GridBagConstraints.BOTH;
	constraints.anchor = GridBagConstraints.CENTER; 

	/** Right pane: codeScrollPane */

	/** filling the code table */
        /* !!!
	codeTable = new Object[startGUI.getProgramSize()/4][2];
	// !!! try {
	    for (int i=0; i<startGUI.getProgramSize()/4; i++) {
		codeTable[i][0] = new Integer(i*4);
		codeTable[i][1] = startGUI.writer.getInstructionString(startGUI.getEmulator().getCodeAt(i*4));
	    } // for
        */
	// !!! } catch (BadLocationException e) {
	// !!!     popupError("Error while filling the instr table");
	// !!! }

	/** preparing the columnNames */
	columnNames = new String[2];
	columnNames[0] = "#";
	columnNames[1] = "Instruction";

	/** Constructing two renderers */
	final DefaultTableCellRenderer highlightRenderer =
            new DefaultTableCellRenderer();
	highlightRenderer.setBackground(Color.yellow);

	final DefaultTableCellRenderer breakPointRenderer =
            new DefaultTableCellRenderer();
	breakPointRenderer.setBackground(new Color(232,89,84));

	final DefaultTableCellRenderer normalRenderer =
            new DefaultTableCellRenderer();
	normalRenderer.setBackground(Color.white);
	
	/** turn the RiscTable into a JTable */
        model = new MyTableModel(); // !!!
	codeJTable = new JTable(model) {
                public TableCellRenderer getCellRenderer(int row, int column) {
		    //	    System.out.println("row: " + row);
		    if (column == 0 && startGUI.getEmulator().isBP_(4 * row)) {
			return breakPointRenderer;
		    } else if (row == startGUI.getEmulator().getPC()/4) {
                        return highlightRenderer;
                    } else {
			return normalRenderer;
		    }
                }
            };
        codeJTable.setFont(new Font("Monospaced", Font.PLAIN, codeJTable.getFont().getSize()));

	/** Setting the preferred size for the columns */
	TableColumn column = null;
	column = codeJTable.getColumnModel().getColumn(0);
	column.setPreferredWidth(30);
	column.setMaxWidth(40);

	/** Creating the ScrollPane */
	codeScrollPane = new JScrollPane(codeJTable);
	codeJTable.setPreferredScrollableViewportSize(new Dimension(80, 500));

	/** puttin the scrollPane in the mainFrame */
	constraints.gridx = 1;
	constraints.gridy = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
	frame.getContentPane().add(codeScrollPane, constraints);
	
	/** Table */
	codeJTable.addMouseListener(new MouseListener() {
	    public void mouseClicked(MouseEvent e) {
		int col = codeJTable.getSelectedColumn();
		int row = codeJTable.getSelectedRow();
		if (col == 0 && row != -1) {
		    startGUI.getEmulator().toggleBP_(row*4);
		    codeJTable.clearSelection();
                    model.fireTableRowsUpdated(row, row);
		}
		    
	    } // mouseClicked

	    public void mouseEntered(MouseEvent e) {}
	    public void mousePressed(MouseEvent e) {}
	    public void mouseExited(MouseEvent e) {}
	    public void mouseReleased(MouseEvent e) {}
	    
	});


	/** setting the frame visible */
	// !!! frame.setVisible(true);
	
    } // AsmCode constructor

    //########################################################################

    public void update() {
        int new_size = size() / 4;
        int old_size = model.row_count;
        model.fireTableRowsUpdated(0, Math.min(new_size, old_size) - 1);
        model.row_count = new_size;
        if (new_size > old_size) {
            model.fireTableRowsInserted(old_size, new_size - 1);
        }
        if (new_size < old_size) {
            model.fireTableRowsDeleted(new_size, old_size - 1);
        }
    }

    private final MyTableModel model;

    private Debugger data() {
        return startGUI.getEmulator();
    }

    private CodeWriter writer() {
        return startGUI.writer;
    }

    private int size() {
        return startGUI.getProgramSize();
    }

    private class MyTableModel extends AbstractTableModel {

        public int row_count = 0;

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int col) {
            switch (col) {
            case 0: return "#";
            case 1: return "Instruction";
            default: throw new Error("col = " + col);
            }
        }

        public int getRowCount() {
            return row_count;
        }

        public Object getValueAt(int row, int col) {
            int adr = 4 * row;
            switch (col) {
            case 0: return "" + adr;
            case 1:return writer().getInstructionString(data().getCodeAt(adr));
            default: throw new Error("col = " + col);
            }
        }

    }

    //########################################################################
}
