package risc.gui;

/**
 * Display of the assembly code
 *
 * 01.05.01 by Benjamin Leroy-Beaulieu
 *
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.*;

import risc.debugger.Function;

public class FunctionGUI {
    
    public JFrame         frame;

    private JScrollPane    scrollPanel;
    private Object[][]     functionTable;
    private String[]       columnNames;
    private RiscTable      functionRiscTable;
    private JTable         functionJTable;

    private JButton        displayRegistersButton;

    /** Pointer to the main gui */
    private StartGUI       startGUI;

    /** utils for the maintain of the table */
    private int            functionTableSize;
    
    
    public FunctionGUI(StartGUI sGUI) {

	startGUI = sGUI;

	/** Creating the frame */
	Toolkit tk  = Toolkit.getDefaultToolkit();
	Dimension d = tk.getScreenSize();
	int screenw = d.width;
	int screenh = d.height;
	frame = new JFrame("Function Stack");
	frame.setSize(2*screenw/10, 2*screenh/3);
	frame.setLocation(0, screenh/3);

	frame.setFont(GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts()[0]);

	/** Allowing window closing */
        frame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
		
	/** frame's layout Manager */
	frame.getContentPane().setLayout(new GridBagLayout());

	/** frame's constraint object */ 
        GridBagConstraints constraints = new GridBagConstraints();
	// components grow in both directions
        constraints.fill    = GridBagConstraints.BOTH; 
	// 5-pixel margin on all sides
        constraints.insets  = new Insets(5,5,5,5); 

	/**
	 * The table
	 */

	/** filling the functionTable */
	functionTableSize = startGUI.getEmulator().getTraceSize();
	functionTable = new Object[functionTableSize][3];
	for (int i=0; i<functionTableSize; i++) {
	    functionTable[i][0] = startGUI.getEmulator().getTraceAt(i).getName();
	    functionTable[i][1] = new Integer(startGUI.getEmulator().getTraceAt(i).getAddress());
	    // the next instr is fucking wrong.
	    functionTable[i][2] = new Integer(((Function)startGUI.getEmulator().getTraceAt(i)).getCallerAddress());
	}

	/** preparing the columnNames */
	columnNames = new String[3];
	columnNames[0] = "Name";
	columnNames[1] = "Address";
	columnNames[2] = "Called from address";

	/** filling the functionRiscTable */
	functionRiscTable = new RiscTable(columnNames, functionTable);

	/** turn it into a JTable */
	functionJTable = new JTable(functionRiscTable);
	functionJTable.setPreferredScrollableViewportSize(new Dimension(80, 300));
	
	/** put the table in a scrollPane */
	scrollPanel = new JScrollPane(functionJTable);
	
	/** put the scrollPanel in the mainFrame */
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
	frame.getContentPane().add(scrollPanel, constraints);
	

	/**
	 * Now, the button
	 */

	displayRegistersButton = new JButton("Display Registers");
	displayRegistersButton.setMnemonic(KeyEvent.VK_D);
	constraints.gridx = 0;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 0.0;
        constraints.weighty = 0.0;
	frame.getContentPane().add(displayRegistersButton, constraints);

	/**
	 * the action listener
	 */

	displayRegistersButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		int numberOfSelectedRows, firstSelectedRow;
		firstSelectedRow = functionJTable.getSelectedRow();
		numberOfSelectedRows = functionJTable.getSelectedRowCount();
		for(int i=firstSelectedRow; i<firstSelectedRow + numberOfSelectedRows; i++)
		    new FunctionRegisterGUI(startGUI, startGUI.getEmulator().getTraceAt(i), startGUI.getEmulator());
	    } // actionPerformed
	});
	
	
	// !!! frame.setVisible(true);

    } // constructor FunctionStackGUI

    protected void updateData() {
	/** refill the functionTable */
	functionTableSize = startGUI.getEmulator().getTraceSize();
	functionTable = new Object[functionTableSize][3];
	for (int i=0; i<functionTableSize; i++) {
	    functionTable[i][0] = startGUI.getEmulator().getTraceAt(i).getName();
	    functionTable[i][1] = new Integer(startGUI.getEmulator().getTraceAt(i).getAddress());
	    //functionTable[i][2] = new Integer(startGUI.getEmulator().getTraceAt(i).getRAt(startGUI.getEmulator().getLinkRegister())/4 -1);
	    functionTable[i][2] = new Integer(((Function)startGUI.getEmulator().getTraceAt(i)).getCallerAddress());
	}
	functionRiscTable.update(columnNames, functionTable);
    }
    
} // class FunctionGUI
