package risc.gui;

/**
 * Display of the registers
 *
 * 30.04.01 by Benjamin Leroy-Beaulieu
 *
 */

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.TableColumn;
import java.lang.Math;

import risc.RISC;
import risc.emulator.Emulator;
import risc.debugger.Function;

public class FunctionRegisterGUI {

    protected JFrame         frame;
    private Object[][]       registersValuesTable = new Object[RISC.register_count][2]; 
    private String[]         columnNames = new String[2];
    private RiscTable        registersTable;
    private JTable           displayJTable;
    private JScrollPane      scrollPane;

    /** pointer to the main startGUI */
    private StartGUI         startGUI;
    private Function         myFunction;
    private Emulator         myEmu;

    public FunctionRegisterGUI(StartGUI gui, Function f, Emulator emu) {
	startGUI = gui;
	myFunction = f;
	myEmu = emu;
	
	/** Creating the frame */
	Toolkit tk  = Toolkit.getDefaultToolkit();
	Dimension d = tk.getScreenSize();
	int screenw = d.width;
	int screenh = d.height;
	frame = new JFrame(myFunction.getName());
	frame.setSize(240,600);
	frame.setLocation((int)(Math.random()*(screenw-240)), (int)(Math.random()*(screenh-600)));

	/** Allowing window closing */
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {}
	});

	/** Layout Manager */
	frame.getContentPane().setLayout(new GridBagLayout());

	/** Constraint object */  
        GridBagConstraints constraints = new GridBagConstraints();
	// components grow only horitontaly 
        constraints.fill    = GridBagConstraints.BOTH; 
	// 5-pixel margin on all sides
        constraints.insets  = new Insets(5,5,5,5); 
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;   

	// filling the registers
	for (int i=0; i<RISC.register_count; i++) {
	    //  registers[i] = i;
	    registersValuesTable[i][0] = startGUI.writer.getRegisterName(i);
	    registersValuesTable[i][1] = new Integer(myFunction.getRAt(i));
	} // for

	/** preparing the columnNames */
	columnNames[0] = "Reg";
	columnNames[1] = "Content";

	/** filling the registerTable */
	registersTable = new RiscTable(columnNames, registersValuesTable);

	/** turn the registerTable into a JTable */
	displayJTable = new JTable(registersTable);

	/** Setting the preferred size for the columns */
	TableColumn column = null;
	column = displayJTable.getColumnModel().getColumn(0);
	column.setPreferredWidth(30);
	column.setMaxWidth(40);

	/** putting the JTable in a scrollPane */
	scrollPane = new JScrollPane(displayJTable);
	displayJTable.setPreferredScrollableViewportSize(new Dimension(80, 300));

	/** Putting the scrollPane in the main frame */
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	frame.getContentPane().add(scrollPane, constraints);

	frame.setVisible(true);

    } // Registers contructor

} // Registers class
