package risc.gui;

/**
 * Login file for the risc debugger
 * 
 * 29.04.01 by Benjamin Leroy-Beaulieu
 *
 */

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.filechooser.*;

import risc.RISC;
import risc.emulator.Emulator;
import risc.emulator.Pipeline;
import risc.CodeWriter;
import risc.debugger.Debugger;

import risc.syscall.Dispatcher;
import risc.syscall.Dispatchee;
import risc.syscall.InputOutput;
import risc.syscall.GarbageCollector;
import risc.syscall.Miscellaneous;
import risc.syscall.Debug;

public class StartGUI {

    //########################################################################
    // Private state

    // Event listener
    private final Listener
        listener          = new Listener();

    // File menu items
    private final JMenuItem
        miExit            = new JMenuItem("Exit", ke.VK_X);

    // Debug menu items
    private final JMenuItem
        miLoad            = new JMenuItem("Load Program..."   , ke.VK_L),
        miRun             = new JMenuItem("Run"               , ke.VK_R),
        miRunWithoutBPs   = new JMenuItem("Run without breaks", ke.VK_W),
        miStepIn          = new JMenuItem("Step In"           , ke.VK_I),
        miStepOver        = new JMenuItem("Step Over"         , ke.VK_V),
        miStepOut         = new JMenuItem("Step Out"          , ke.VK_O),
        miStop            = new JMenuItem("Stop"              , ke.VK_S),
        miRestart         = new JMenuItem("Restart"           , ke.VK_T);

    // View menu items
    private final JMenuItem
        miShowAll         = new JMenuItem        ("Show All"      , ke.VK_S),
        miHideAll         = new JMenuItem        ("Hide All"      , ke.VK_H),
        miToggleRegisters = new_JCheckBoxMenuItem("Show Registers", ke.VK_R),
        miToggleMemory    = new_JCheckBoxMenuItem("Show Memory"   , ke.VK_M),
        miToggleCode      = new_JCheckBoxMenuItem("Show Code"     , ke.VK_C),
        miToggleStack     = new_JCheckBoxMenuItem("Show Stack"    , ke.VK_K),
        miToggleTrace     = new_JCheckBoxMenuItem("Show Trace"    , ke.VK_T);

    // Menu bar menus
    private final JMenu
        mnFile            = new_JMenu("File" , ke.VK_F),
        mnDebug           = new_JMenu("Debug", ke.VK_D),
        mnView            = new_JMenu("View" , ke.VK_W);

    // Tool bar buttons
    private final JButton
        btRun             = new_JButton("Run"      , ke.VK_R),
        btStepIn          = new_JButton("Step In"  , ke.VK_I),
        btStepOver        = new_JButton("Step Over", ke.VK_V),
        btStepOut         = new_JButton("Step Out" , ke.VK_O),
        btStop            = new_JButton("Stop"     , ke.VK_S);

    // File chooser
    private final JFileChooser fcLoad; // !!!

    // Text area
    private final JTextArea taMessages; // !!!

    // !!!
    private final JLabel statusbar = new JLabel();

    //########################################################################
    // Constructors

    public StartGUI(int memsize, 
					boolean verbose, 
					String filename, 
					Pipeline pipeline,
					boolean trace) {
        // Load menu item
        miLoad.setIcon(new ImageIcon("images/open.gif")); // !!!

        // File menu
        mnFile.add(init(miExit            , listener, ksExit));

        // Debug menu
        mnDebug.add(init(miLoad           , listener, ksLoad));
	mnDebug.addSeparator();
        mnDebug.add(init(miRun            , listener, ksRun));
        mnDebug.add(init(miRunWithoutBPs  , listener, ksRunWithoutBPs));
	mnDebug.addSeparator();
        mnDebug.add(init(miStepIn         , listener, ksStepIn));
        mnDebug.add(init(miStepOver       , listener, ksStepOver));
        mnDebug.add(init(miStepOut        , listener, ksStepOut));
	mnDebug.addSeparator();
        mnDebug.add(init(miStop           , listener, ksStop));
        mnDebug.add(init(miRestart        , listener, ksRestart));

        // View menu
        mnView.add(init(miShowAll         , listener, ksShowAll));
        mnView.add(init(miHideAll         , listener, ksHideAll));
	mnView.addSeparator();
        mnView.add(init(miToggleRegisters , listener, ksToggleRegisters));
        mnView.add(init(miToggleMemory    , listener, ksToggleMemory));
        mnView.add(init(miToggleCode      , listener, ksToggleCode));
        mnView.add(init(miToggleStack     , listener, ksToggleStack));
        mnView.add(init(miToggleTrace     , listener, ksToggleTrace));

        // Menu bar
        JMenuBar menu = new JMenuBar();
        menu.add(mnFile);
        menu.add(mnDebug);
        menu.add(mnView);

        // Tool bar
        JPanel toolbar = new JPanel();
        toolbar.setLayout(new GridLayout(1, 0, 5, 5));
        toolbar.add(init(btRun     , listener));
        toolbar.add(init(btStepIn  , listener));
        toolbar.add(init(btStepOver, listener));
        toolbar.add(init(btStepOut , listener));
        toolbar.add(init(btStop    , listener));

	// File chooser
	this.fcLoad = new JFileChooser(System.getProperty("user.dir"));
	fcLoad.setFileFilter(new AsmFileFilter());

        // Text area
	this.taMessages = new JTextArea(24, 40);
 	taMessages.setEditable(false);
        JScrollPane spMessages = new JScrollPane(taMessages);

        // !!!
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.fill       = GridBagConstraints.BOTH;
        constraints.insets     = new Insets(5,5,5,5);
	constraints.gridx      = 0;
	constraints.gridwidth  = 1;
	constraints.gridheight = 1;
        constraints.weightx    = 1.0;

	this.frame = new JFrame("Simple RISC Debugger");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // !!!
	frame.setJMenuBar(menu);
	frame.getContentPane().setLayout(new GridBagLayout());
	constraints.gridy      = 0;
        constraints.weighty    = 0.0;
	frame.getContentPane().add(toolbar, constraints);
	constraints.gridy      = 1;
        constraints.weighty    = 1.0;
	frame.getContentPane().add(spMessages, constraints);
	constraints.gridy      = 2;
        constraints.weighty    = 0.0;
	frame.getContentPane().add(statusbar, constraints);

        // !!!
        statusbar.setText("Staring up ...");

	/**
	 * setting the frame to be visible
	 */
	Toolkit tk  = Toolkit.getDefaultToolkit();
	Dimension d = tk.getScreenSize();
	int screenw = d.width;
	int screenh = d.height;
	frame.setSize(screenw/2,screenh/3);
	frame.setLocation(0, 0);
	frame.setVisible(true);

        MyWriter writer = new MyWriter();
        Dispatchee io = new InputOutput(
            new BufferedReader(new InputStreamReader(System.in)),
            new PrintWriter(writer));
        Dispatchee gc = new GarbageCollector(verbose, writer);
        Dispatchee db = new Debug();
        Dispatchee misc = new Miscellaneous();
        Dispatcher os = new Dispatcher(new Dispatchee[] {io, gc, db, misc});

        emu = new Debugger(memsize, os, pipeline, trace);

        // Child windows
        this.registers = new RegistersGUI(StartGUI.this);
        this.memory = new MemoryGUI(StartGUI.this);
        this.asmCode = new AsmCode(StartGUI.this);
        this.stack = new StackGUI(StartGUI.this);
        this.functionStack = new FunctionGUI(StartGUI.this);

	/**
	 * open the file given as parameter
	 */
	if (filename != null) {
	    asmFile = new File(filename);
	    loadFile(false);
	    // !!! fileToLoadTextField.setText(fileString);
	}
    } // StartGUI contructor

    //########################################################################
    //########################################################################
    //########################################################################
    //########################################################################




    /** other windows pointers */
    private final RegistersGUI registers;
    private final AsmCode      asmCode;
    private final StackGUI     stack;
    private final MemoryGUI    memory;
    private final FunctionGUI  functionStack;
    // private MessageGUI   messagegui; // !!!

    private final Debugger    emu;

    //########################################################################

    private class Listener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();
            if (source == miExit           ) doExit           ();
            if (source == miLoad           ) doLoad           ();
            if (source == miRun            ) doRun            ();
            if (source == miRunWithoutBPs  ) doRunWithoutBPs  ();
            if (source == miStepIn         ) doStepIn         ();
            if (source == miStepOver       ) doStepOver       ();
            if (source == miStepOut        ) doStepOut        ();
            if (source == miStop           ) doStop           ();
            if (source == miRestart        ) doRestart        ();
            if (source == miShowAll        ) doShowAll        ();
            if (source == miHideAll        ) doHideAll        ();
            if (source == miToggleRegisters) doToggleRegisters();
            if (source == miToggleMemory   ) doToggleMemory   ();
            if (source == miToggleCode     ) doToggleCode     ();
            if (source == miToggleStack    ) doToggleStack    ();
            if (source == miToggleTrace    ) doToggleTrace    ();
            if (source == btRun            ) doRun            ();
            if (source == btStepIn         ) doStepIn         ();
            if (source == btStepOver       ) doStepOver       ();
            if (source == btStepOut        ) doStepOut        ();
            if (source == btStop           ) doStop           ();
        }
    }

    //########################################################################

    protected JFrame    frame;

    

    /** Emulator utils */
    private File        asmFile;

    protected void updateWindows() {
        // !!! != null with isVisible()
	if (registers != null) 
	    registers.updateData();
	if (asmCode != null)
	    asmCode.update();
	if (stack != null)
	    stack.update();
	if (memory != null)
	    memory.updateData();
	if (functionStack != null)
	    functionStack.updateData();
    }

    protected Debugger getEmulator() {
	return emu;
    }

    protected void loadFile(boolean restart) {
        statusbar.setText((restart ? "Reloading" : "Loading") + " " +
            asmFile.getName());
        emu.reset();
        if (restart) emu.enableBPs(false); else emu.removeBPs();
	try {
            programSize = emu.assemble(new FileReader(asmFile));
	    statusbar.setText("Ready");
	} catch (FileNotFoundException ex) {
            // !!! bell ?
	    statusbar.setText("File not found: " + asmFile.getName());
	}
        if (restart) emu.enableBPs(true);
        this.restart = false;
        updateWindows();
    }

    protected void printMessage(String message) {
	taMessages.append(message + "\n");
	String text = taMessages.getText();
	taMessages.setCaretPosition(text.length());
    }

    protected void printMessage() {
	this.printMessage("");
    }	

    protected File getAsmFile() {
	return asmFile;
    }

    private void printError(String em) {
	JOptionPane.showMessageDialog(null, 
				      em,
				      "Error",
				      JOptionPane.ERROR_MESSAGE);	
    }

    //########################################################################
    // !!!

    private static final KeyEvent  ke = null;
    private static final KeyStroke ks = null;

    public static final KeyStroke
        ksExit            = ks.getKeyStroke(ke.VK_Q, Event.CTRL_MASK),
        ksLoad            = null,
        ksRun             = null,
        ksRunWithoutBPs   = null,
        ksStepIn          = null,
        ksStepOver        = null,
        ksStepOut         = null,
        ksStop            = null,
        ksRestart         = null,
        ksShowAll         = ks.getKeyStroke(ke.VK_S, Event.CTRL_MASK),
        ksHideAll         = ks.getKeyStroke(ke.VK_H, Event.CTRL_MASK),
        ksToggleRegisters = ks.getKeyStroke(ke.VK_R, Event.CTRL_MASK),
        ksToggleMemory    = ks.getKeyStroke(ke.VK_M, Event.CTRL_MASK),
        ksToggleCode      = ks.getKeyStroke(ke.VK_C, Event.CTRL_MASK),
        ksToggleStack     = ks.getKeyStroke(ke.VK_K, Event.CTRL_MASK),
        ksToggleTrace     = ks.getKeyStroke(ke.VK_T, Event.CTRL_MASK);

    private static JMenuItem new_JCheckBoxMenuItem(String text, int mnemonic) {
        JMenuItem item = new JCheckBoxMenuItem(text);
        item.setMnemonic(mnemonic);
        return item;
    }

    private static JMenu new_JMenu(String text, int mnemonic) {
        JMenu menu = new JMenu(text);
        menu.setMnemonic(mnemonic);
        return menu;
    }

    private static JButton new_JButton(String text, int mnemonic) {
        JButton button = new JButton(text);
        button.setMnemonic(mnemonic);
        return button;
    }

    public static JMenuItem init(JMenuItem item, ActionListener listener,
        KeyStroke accelerator)
    {
        item.addActionListener(listener);
        if (accelerator != null) item.setAccelerator(accelerator);
        return item;
    }

    public static JButton init(JButton button, ActionListener listener) {
        button.addActionListener(listener);
        return button;
    }

    //########################################################################
    // !!!

    public void doExit() {
        System.exit(0);
    }

    public void doLoad() {
        int returnVal = fcLoad.showOpenDialog(frame);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            asmFile = fcLoad.getSelectedFile();
            // !!! fileToLoadTextField.setText(asmFile.getName());
            loadFile(false);
        } else {
            // !!! fileToLoadTextField.setText("");
        }
    }

    public void doRun() {
        run('b');
    }

    public void doRunWithoutBPs() {
        run('t');
    }

    public void doStepIn() {
        run('i');
    }

    public void doStepOver() {
        run('v');
    }

    public void doStepOut() {
        run('o');
    }

    public void doStop() {
        if (running) emu.stop();
    }

    public void doRestart() {
        loadFile(true);
    }

    public void doShowAll() {
        doShow(registers.frame);
        doShow(memory.frame);
        doShow(asmCode.frame);
        doShow(stack.frame);
        doShow(functionStack.frame);
    }

    public void doHideAll() {
        doHide(registers.frame);
        doHide(memory.frame);
        doHide(asmCode.frame);
        doHide(stack.frame);
        doHide(functionStack.frame);
    }

    public void doToggleRegisters() {
        doToggle(registers.frame);
    }

    public void doToggleMemory() {
        doToggle(memory.frame);
    }

    public void doToggleCode() {
        doToggle(asmCode.frame);
    }

    public void doToggleStack() {
        doToggle(stack.frame);
    }

    public void doToggleTrace() {
        doToggle(functionStack.frame);
    }

    public void doToggle(JFrame frame) {
        if (frame.isVisible()) doHide(frame); else doShow(frame);
    }

    public void doShow(JFrame frame) {
        frame.setState(Frame.NORMAL);
        frame.setVisible(true);
    }

    public void doHide(JFrame frame) {
        frame.setVisible(false);
    }

    //########################################################################
    // !!!

    //########################################################################
    // !!!

    public CodeWriter writer = new CodeWriter();//!!!
    // !!! public int RET = RISC.R28;
    /** Number of the register representing the frame pointer */
    public int FP = RISC.R29;
    /** Number of the register representing the stack pointer */
    public int SP = RISC.R30;
    /** Number of the register representing the saved program counter */
    public int LNK = RISC.LNK; // !!! = CP

    private class MyWriter extends Writer {

        public void close() {}
        public void flush() {}

        public void write(char[] cbuf, int off, int len) {
            // !!! check
            taMessages.append(new String(cbuf, off, len));
            taMessages.setCaretPosition(taMessages.getText().length());
        }
    }

    /** Program size */
    private int programSize;

    /**
     * returns the program size.
     */
    public int getProgramSize() {
	return programSize;
    }

    private boolean running = false;
    private boolean restart = false;

    private synchronized void run(final char mode) {
        if (restart) doRestart();
        if (!running) {
            running = true;
            new Thread() { public void run() { run1(mode); }}.start();
        }
    }

    private void run1(char mode) {
        statusbar.setText("running");
        // !!! start new thread ?
        String status = "stopped";
        try {
            run2(mode);
            status =  "stopped"; // !!! correct ?
        } catch (Emulator.Interrupt exception) {
            status =  "stopped";
        } catch (Emulator.Break exception) {
            status =  "stopped";
        } catch (Emulator.Exit exception) {
            int code = exception.code;
            status = "terminated " +
                (code != 0 ? "with exit code " + code : "successfully");
            restart = true;
            // !!! duplicated code
        } catch (Emulator.Error exception) {
            status = "error: "  + exception.getMessage();
        } catch (Emulator.Exception exception) {
            status = "error: "  + exception.toString();
        }
        statusbar.setText(status);
        updateWindows();
        running = false;
    }

    private void run2(char mode) throws Emulator.Exception {
        try {
            switch(mode) {
            case 'i': emu.stepIn(); return;
            case 'v': emu.stepOver(); return;
            case 'o': emu.stepOut(); return;
            case 'b': emu.run(); return;
            case 't': emu.runThrough(); return;
            default: throw new Error("Unknown run mode: " + mode);
            }
        } finally {
            emu.syscall_nofail(RISC.SYS_IO_FLUSH, 0, 0); // !!!
        }
    }

} // StartGUI class
